@@ -64,10 +64,10 @@ imap_f(int argc, char **argv)
 void
 imap_init(void)
 {
-	imap_cmd.name = _("imap");
+	imap_cmd.name = "imap";
 	imap_cmd.cfunc = imap_f;
 	imap_cmd.argmin = 0;
-	imap_cmd.argmax = 0;
+	imap_cmd.argmax = 1;
 	imap_cmd.args = _("[nentries]");
 	imap_cmd.flags = CMD_NOMAP_OK;
 	imap_cmd.oneline = _("inode map for filesystem of current file");
