@@ -29,6 +29,7 @@ static cmdinfo_t mread_cmd;
 static cmdinfo_t msync_cmd;
 static cmdinfo_t munmap_cmd;
 static cmdinfo_t mwrite_cmd;
+static cmdinfo_t mremap_cmd;
 
 mmap_region_t	*maptable;
 int		mapcount;
@@ -336,7 +337,7 @@ mread_help(void)
 " Accesses a range of the current memory mapping, optionally dumping it to\n"
 " the standard output stream (with -v option) for subsequent inspection.\n"
 " -f -- verbose mode, dump bytes with offsets relative to start of file.\n"
-" -r -- reverse order; start accessing fom the end of range, moving backward\n"
+" -r -- reverse order; start accessing from the end of range, moving backward\n"
 " -v -- verbose mode, dump bytes with offsets relative to start of mapping.\n"
 " The accesses are performed sequentially from the start offset by default.\n"
 " Notes:\n"
@@ -500,7 +501,7 @@ mwrite_help(void)
 " Stores a byte into memory for a range within a mapping.\n"
 " The default stored value is 'X', repeated to fill the range specified.\n"
 " -S -- use an alternate seed character\n"
-" -r -- reverse order; start storing fom the end of range, moving backward\n"
+" -r -- reverse order; start storing from the end of range, moving backward\n"
 " The stores are performed sequentially from the start offset by default.\n"
 "\n"));
 }
@@ -574,11 +575,71 @@ mwrite_f(
 	return 0;
 }
 
+static void
+mremap_help(void)
+{
+	printf(_(
+"\n"
+" resizes the current memory mapping\n"
+"\n"
+" Examples:\n"
+" 'mremap 8192' - resizes the current mapping to 8192 bytes.\n"
+"\n"
+" Resizes the mappping, growing or shrinking from the current size.\n"
+" The default stored value is 'X', repeated to fill the range specified.\n"
+" -f -- use the MREMAP_FIXED flag\n"
+" -m -- use the MREMAP_MAYMOVE flag\n"
+"\n"));
+}
+
+int
+mremap_f(
+	int		argc,
+	char		**argv)
+{
+	ssize_t		new_length;
+	void		*new_addr;
+	int		flags = 0;
+	int		c;
+	size_t		blocksize, sectsize;
+
+	while ((c = getopt(argc, argv, "fm")) != EOF) {
+		switch (c) {
+		case 'f':
+			flags = MREMAP_FIXED|MREMAP_MAYMOVE;
+			break;
+		case 'm':
+			flags = MREMAP_MAYMOVE;
+			break;
+		default:
+			return command_usage(&mremap_cmd);
+		}
+	}
+
+	init_cvtnum(&blocksize, &sectsize);
+	new_length = cvtnum(blocksize, sectsize, argv[optind]);
+	if (new_length < 0) {
+		printf(_("non-numeric offset argument -- %s\n"),
+			argv[optind]);
+		return 0;
+	}
+
+	new_addr = mremap(mapping->addr, mapping->length, new_length, flags);
+	if (new_addr == MAP_FAILED)
+		perror("mremap");
+	else {
+		mapping->addr = new_addr;
+		mapping->length = new_length;
+	}
+
+	return 0;
+}
+
 void
 mmap_init(void)
 {
-	mmap_cmd.name = _("mmap");
-	mmap_cmd.altname = _("mm");
+	mmap_cmd.name = "mmap";
+	mmap_cmd.altname = "mm";
 	mmap_cmd.cfunc = mmap_f;
 	mmap_cmd.argmin = 0;
 	mmap_cmd.argmax = -1;
@@ -588,8 +649,8 @@ mmap_init(void)
 		_("mmap a range in the current file, show mappings");
 	mmap_cmd.help = mmap_help;
 
-	mread_cmd.name = _("mread");
-	mread_cmd.altname = _("mr");
+	mread_cmd.name = "mread";
+	mread_cmd.altname = "mr";
 	mread_cmd.cfunc = mread_f;
 	mread_cmd.argmin = 0;
 	mread_cmd.argmax = -1;
@@ -599,8 +660,8 @@ mmap_init(void)
 		_("reads data from a region in the current memory mapping");
 	mread_cmd.help = mread_help;
 
-	msync_cmd.name = _("msync");
-	msync_cmd.altname = _("ms");
+	msync_cmd.name = "msync";
+	msync_cmd.altname = "ms";
 	msync_cmd.cfunc = msync_f;
 	msync_cmd.argmin = 0;
 	msync_cmd.argmax = -1;
@@ -609,16 +670,16 @@ mmap_init(void)
 	msync_cmd.oneline = _("flush a region in the current memory mapping");
 	msync_cmd.help = msync_help;
 
-	munmap_cmd.name = _("munmap");
-	munmap_cmd.altname = _("mu");
+	munmap_cmd.name = "munmap";
+	munmap_cmd.altname = "mu";
 	munmap_cmd.cfunc = munmap_f;
 	munmap_cmd.argmin = 0;
 	munmap_cmd.argmax = 0;
 	munmap_cmd.flags = CMD_NOFILE_OK | CMD_FOREIGN_OK;
 	munmap_cmd.oneline = _("unmaps the current memory mapping");
 
-	mwrite_cmd.name = _("mwrite");
-	mwrite_cmd.altname = _("mw");
+	mwrite_cmd.name = "mwrite";
+	mwrite_cmd.altname = "mw";
 	mwrite_cmd.cfunc = mwrite_f;
 	mwrite_cmd.argmin = 0;
 	mwrite_cmd.argmax = -1;
@@ -628,9 +689,21 @@ mmap_init(void)
 		_("writes data into a region in the current memory mapping");
 	mwrite_cmd.help = mwrite_help;
 
+	mremap_cmd.name = "mremap";
+	mremap_cmd.altname = "mrm";
+	mremap_cmd.cfunc = mremap_f;
+	mremap_cmd.argmin = 1;
+	mremap_cmd.argmax = 2;
+	mremap_cmd.flags = CMD_NOFILE_OK | CMD_FOREIGN_OK;
+	mremap_cmd.args = _("[-m|-f] newsize");
+	mremap_cmd.oneline =
+		_("alters the size of the current memory mapping");
+	mremap_cmd.help = mremap_help;
+
 	add_command(&mmap_cmd);
 	add_command(&mread_cmd);
 	add_command(&msync_cmd);
 	add_command(&munmap_cmd);
 	add_command(&mwrite_cmd);
+	add_command(&mremap_cmd);
 }
