@@ -595,7 +595,6 @@ libxfs_alloc_file_space(
 			break;
 		}
 		xfs_trans_ijoin(tp, ip, 0);
-		xfs_trans_ihold(tp, ip);
 
 		xfs_bmap_init(&free_list, &firstfsb);
 		error = xfs_bmapi_write(tp, ip, startoffset_fsb, allocatesize_fsb,
@@ -701,8 +700,6 @@ libxfs_inode_alloc(
 		if (error)
 			return error;
 	}
-	if (!ip)
-		error = ENOSPC;
 
 	*ipp = ip;
 	return error;
@@ -719,6 +716,7 @@ libxfs_fs_repair_cmn_err(int level, xfs_mount_t *mp, char *fmt, ...)
 	va_start(ap, fmt);
 	vfprintf(stderr, fmt, ap);
 	fprintf(stderr, "  This is a bug.\n");
+	fprintf(stderr, "%s version %s\n", progname, VERSION);
 	fprintf(stderr, "Please capture the filesystem metadata with "
 			"xfs_metadump and\nreport it to xfs@oss.sgi.com.\n");
 	va_end(ap);
